import math
from typing import Tuple
import numpy as np
import pandas as pd


Row = pd.Series


def _distance_py(station: Row, city: Row) -> float:
    # adapted from http://stackoverflow.com/a/21623206/1236542
    lat1 = station['latitude']
    lon1 = station['longitude']
    lat2 = city['latitude']
    lon2 = city['longitude']

    p = 0.017453292519943295
    a = 0.5 - math.cos((lat2 - lat1) * p) / 2 + math.cos(lat1 * p) * math.cos(lat2 * p) * (1 - math.cos((lon2 - lon1) * p)) / 2
    dist = 1000 * 12742 * math.asin(np.sqrt(a))

    return dist
    

def distance_apply(stations: pd.DataFrame, city: Row) -> pd.Series:
    dists = stations.apply(_distance_py, axis=1, city=city)
    return dists


def distance(stations: pd.DataFrame, city: Row) -> pd.Series:
    """
    Return distances between points, in metres
    """
    # adapted from http://stackoverflow.com/a/21623206/1236542
    lat1 = stations['latitude']
    lon1 = stations['longitude']
    lat2 = city['latitude']  # a scalar
    lon2 = city['longitude'] # a scalar

    p = 0.017453292519943295
    a = 0.5 - np.cos((lat2 - lat1) * p) / 2 + np.cos(lat1 * p) * np.cos(lat2 * p) * (1 - np.cos((lon2 - lon1) * p)) / 2
    dists = 1000 * 12742 * np.arcsin(np.sqrt(a))

    return dists


def best_tmax(city: Row, stations: pd.DataFrame) -> pd.Series:
    distances = distance(stations, city)
    
    # return value from closest station:
    idx = distances.argmin()
    station = stations['avg_tmax'].iloc[idx]
    return station


def load_data(stations_file: str, cities_file: str) -> Tuple[pd.DataFrame, pd.DataFrame]:
    cities = pd.read_csv(cities_file)
    stations = pd.read_json(stations_file, lines=True)
    stations['avg_tmax'] = stations['avg_tmax']/10

    return cities, stations


def main():
    cities, stations = load_data(stations_file, cities_file)
    tmax = cities.apply(best_tmax, axis=1, stations=stations)
    print(tmax)
    